
# q-expansion of newform 704.2.a.n, downloaded from the LMFDB on 02 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 704
weight = 2
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, -1], [-2, 1], [0, 0], [-1, 0], [-2, 0], [2, 2], [-4, 2], [0, -3], [-2, -2], [-8, 1], [-2, 3], [2, -4], [-4, 2], [-4, 0], [2, -4], [8, -3], [6, -6], [0, -3], [8, 3], [-6, 0], [-8, -2], [-4, 6], [-2, 1], [-2, 3], [-10, 2], [-4, -4], [12, 0], [14, 0], [14, -1], [0, 6], [-4, 0], [6, 3], [-12, 0], [-10, 2], [0, 6], [6, -3], [12, 0], [-8, 6], [-18, 0], [0, -3], [-2, -9], [0, 3], [2, -6], [-2, 4], [-4, -4], [4, -8], [0, -3], [-20, 0], [14, 3], [2, 2], [-16, 6], [18, -6], [0, 9], [2, -4], [8, 0], [10, -8], [-8, -2], [-2, 0], [-6, 0], [-20, 4], [-10, 8], [-12, -6], [-12, 12], [14, -3], [6, 9], [-24, 3], [-22, 6], [-12, -6], [-2, 6], [22, 1], [0, 6], [-24, 9], [6, 6], [-16, -1], [16, -3], [14, 5], [-14, 0], [2, 8], [-14, 0], [-12, 12], [10, 0], [-8, -12], [-2, 3], [-16, -4], [16, -3], [-10, 5], [26, 6], [14, 2], [16, 7], [16, -3], [16, -6], [-24, -3], [-4, 12], [20, 8], [0, 6], [6, 3], [6, -9], [12, -18], [-18, 0], [-12, 0], [-18, -6], [12, -12], [-6, -6], [-12, 0], [-2, 3], [-28, 0], [10, -14], [-4, -12], [-14, 18], [32, 2], [-2, -6], [-22, -4], [0, 3], [0, -15], [-10, -7], [40, 1], [24, 3], [6, 3], [28, -6], [-2, 9], [-6, 12], [-2, -14], [12, 0], [-8, 13], [22, 10], [-18, 3], [24, -3], [24, -3], [-10, 18], [-4, -10], [0, -12], [-16, 11], [10, 0], [-14, 4], [-38, 0], [2, -4], [12, 0], [22, 1], [2, -4], [-36, -6], [30, -12], [16, 1], [12, -6], [-10, -3], [8, -15], [-10, -12], [10, -2], [-40, -7], [-12, 12], [38, -12], [14, -13], [36, 0], [-24, 18], [-4, 8], [-4, 12], [-8, 10], [2, 8], [10, -12], [-2, 4], [-24, -3], [-22, -10], [0, -12], [-24, 3], [14, -25], [24, -3], [36, -12], [-10, -6]]
